@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model-Tester
 #
-# This software is Copyright (c) 2013-2016 by Dominique Dumont.
+# This software is Copyright (c) 2013-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -9,7 +9,7 @@
 #
 package Config::Model::Tester;
 # ABSTRACT: Test framework for Config::Model
-$Config::Model::Tester::VERSION = '2.057';
+$Config::Model::Tester::VERSION = '3.003';
 use warnings;
 use strict;
 use locale;
@@ -46,13 +46,13 @@ require Exporter;
 $File::Copy::Recursive::DirPerms = 0755;
 
 sub setup_test {
-    my ( $app_to_test, $t_name, $wr_root, $trace, $setup ) = @_;
+    my ( $app_to_test, $t_name, $wr_root, $trace, $t_data ) = @_;
 
     # cleanup before tests
     $wr_root->remove_tree();
     $wr_root->mkpath( { mode => 0755 } );
 
-    $conf_dir =~ s!~/!$home_for_test/! if $conf_dir;
+    $conf_dir =~ s!~/!$home_for_test/! if $conf_dir and $home_for_test;
 
     my $wr_dir    = $wr_root->child('test-' . $t_name);
     my $wr_dir2   = $wr_root->child('test-' . $t_name.'-w');
@@ -61,11 +61,11 @@ sub setup_test {
         if $conf_dir and $conf_file_name;
 
     my $ex_dir = path('t')->child('model_tests.d', "$app_to_test-examples");
-    my $ex_data = $ex_dir->child($t_name);
+    my $ex_data = $ex_dir->child($t_data->{data_from} // $t_name);
 
     my @file_list;
 
-    if ($setup) {
+    if (my $setup = $t_data->{setup}) {
         foreach my $file (keys %$setup) {
             my $map = $setup->{$file} ;
             my $destination_str
@@ -74,7 +74,7 @@ sub setup_test {
             if (not defined $destination_str) {
                 die "$app_to_test $t_name setup error: cannot find destination for test file $file" ;
             }
-            $destination_str =~ s!~/!$home_for_test/!;
+            $destination_str =~ s!~/!$home_for_test/! if $home_for_test;
             my $destination = $wr_dir->child($destination_str) ;
             $destination->parent->mkpath( { mode => 0755 }) ;
             my $data_file = $ex_data->child($file);
@@ -96,8 +96,8 @@ sub setup_test {
     }
     elsif ( $ex_data->exists ) {
         # either one if true if $conf_file is undef
-        die "test data is missing \$conf_dir" unless defined $conf_dir;
-        die "test data is missing \$conf_file" unless defined $conf_file;
+        die "test data is missing global \$conf_dir" unless defined $conf_dir;
+        die "test data is missing global \$conf_file_name" unless defined $conf_file;
 
         # just copy file
         say "file copy ". $ex_data->stringify . '->'. $conf_file->stringify
@@ -165,8 +165,17 @@ sub run_update {
 
     local $Config::Model::Value::nowarning = $args{no_warnings} || $t->{no_warnings} || 0;
 
-    note("updating config with ". join(' ',%args));
-    my $res = $inst->update( from_dir => $dir, %args ) ;
+    my $res ;
+    if (my $uw = delete $args{update_warnings}) {
+        note("updating config with warning check and args: ". join(' ',%args));
+        warnings_like { $res = $inst->update( from_dir => $dir, %args ); } $uw,
+            "Updated configuration with warning check ";
+    }
+    else {
+        note("updating config with no warning check and args: ". join(' ',%args));
+        $res = $inst->update( from_dir => $dir, %args ) ;
+    }
+
     if (defined $ret) {
         is($res,$ret,"updated configuration, got expected return value");
     }
@@ -176,10 +185,10 @@ sub run_update {
 }
 
 sub load_instructions {
-    my ($root,$t,$trace) = @_ ;
+    my ($root,$steps,$trace) = @_ ;
 
-    print "Loading $t->{load}\n" if $trace ;
-    $root->load( $t->{load} );
+    print "Loading $steps\n" if $trace ;
+    $root->load( $steps );
     ok( 1, "load called" );
 }
 
@@ -312,6 +321,27 @@ sub write_data_back {
     ok( 1, "$app_to_test write back done" );
 }
 
+sub check_file_mode {
+    my ($wr_dir, $t) = @_;
+
+    if ($^O eq 'MSWin32' and my $fm = $t->{file_mode}) {
+        note("skipping file mode tests on Windows");
+        return;
+    }
+
+    if (my $fm = $t->{file_mode}) {
+        foreach my $f (keys %$fm) {
+            my $expected_mode = $fm->{$f} ;
+            my $stat = $wr_dir->child($f)->stat;
+            ok($stat ,"stat found file $f");
+            if ($stat) {
+                my $mode = $stat->mode & 07777 ;
+                is($mode, $expected_mode, sprintf("check $f mode (got %o vs %o)",$mode,$expected_mode));
+            }
+        }
+    }
+}
+
 sub check_file_content {
     my ($wr_dir, $t) = @_;
 
@@ -320,7 +350,7 @@ sub check_file_content {
             my $t = $fc->{$f} ;
             my @tests = ref $t eq 'ARRAY' ? @$t : ($t) ;
             foreach my $subtest (@tests) {
-                file_contents_eq_or_diff $wr_dir->child($f)->stringify,  $subtest,
+                file_contents_eq_or_diff $wr_dir->child($f)->stringify,  $subtest, { encoding => 'UTF-8' },
                     "check that $f contains $subtest";
             }
         }
@@ -331,7 +361,7 @@ sub check_file_content {
             my $t = $fc->{$f} ;
             my @tests = ref $t eq 'ARRAY' ? @$t : ($t) ;
             foreach my $subtest (@tests) {
-                file_contents_like $wr_dir->child($f)->stringify,  $subtest,
+                file_contents_like $wr_dir->child($f)->stringify,  $subtest, { encoding => 'UTF-8' },
                     "check that $f matches regexp $subtest";
             }
         }
@@ -342,7 +372,7 @@ sub check_file_content {
             my $t = $fc->{$f} ;
             my @tests = ref $t eq 'ARRAY' ? @$t : ($t) ;
             foreach my $subtest (@tests) {
-                file_contents_unlike $wr_dir->child($f)->stringify,  $subtest,
+                file_contents_unlike $wr_dir->child($f)->stringify,  $subtest, { encoding => 'UTF-8' },
                     "check that $f does not match regexp $subtest";
             }
         }
@@ -369,6 +399,9 @@ sub create_second_instance {
     dircopy( $wr_dir->stringify, $wr_dir2->stringify )
         or die "can't copy from $wr_dir to $wr_dir2: $!";
 
+    my @options;
+    push @options, backend_arg => $t->{backend_arg} if $t->{backend_arg};
+
     my $i2_test = $model->instance(
         root_class_name => $model_to_test,
         root_dir        => $wr_dir2->stringify,
@@ -377,6 +410,7 @@ sub create_second_instance {
         application     => $app_to_test,
         check           => $t->{load_check2} || 'yes',
         config_dir      => $config_dir_override,
+        @options
     );
 
     ok( $i2_test, "Created instance $app_to_test-test-$t_name-w" );
@@ -395,6 +429,7 @@ sub run_model_test {
     undef $conf_file_name ;
     undef $conf_dir ;
     undef $home_for_test ;
+    undef $model_to_test ;
 
     note("Beginning $app_to_test test ($app_to_test_conf)");
 
@@ -441,18 +476,31 @@ sub run_model_test {
         note("Beginning subtest $app_to_test $t_name");
 
         my ($wr_dir, $wr_dir2, $conf_file, $ex_data, @file_list)
-            = setup_test ($app_to_test, $t_name, $wr_root,$trace, $t->{setup});
+            = setup_test ($app_to_test, $t_name, $wr_root,$trace, $t);
 
         write_config_file($conf_dir,$wr_dir,$t);
 
+        my $inst_name = "$app_to_test-" . $t_name;
+
+        die "Duplicated test name $t_name for app $app_to_test\n"
+            if $model->has_instance ($inst_name);
+
+        my @options;
+        push @options, backend_arg => $t->{backend_arg} if $t->{backend_arg};
+
+        # eventually, we may end up with several instances of Dpkg
+        # model in the same process. So we can't play with chdir
         my $inst = $model->instance(
             root_class_name => $model_to_test,
+            # need to keed root_dir to handle config files like
+            # /etc/foo.ini (absolute path, like in /etc/)
             root_dir        => $wr_dir->stringify,
-            instance_name   => "$app_to_test-" . $t_name,
+            instance_name   => $inst_name,
             application     => $app_to_test,
             config_file     => $t->{config_file} ,
             check           => $t->{load_check} || 'yes',
             config_dir      => $config_dir_override,
+            @options
         );
 
         my $root = $inst->config_root;
@@ -461,7 +509,7 @@ sub run_model_test {
 
         run_update($inst,$wr_dir,$t) if $t->{update};
 
-        load_instructions ($root,$t,$trace) if $t->{load} ;
+        load_instructions ($root,$t->{load},$trace) if $t->{load} ;
 
         dump_tree ('before fix '.$app_to_test , $root, 'full', $t->{no_warnings}, $t->{check_before_fix}, $trace)
             if $t->{check_before_fix};
@@ -483,10 +531,14 @@ sub run_model_test {
 
         check_file_content($wr_dir,$t) ;
 
+        check_file_mode($wr_dir,$t) ;
+
         check_added_or_removed_files ($conf_dir, $wr_dir, $t, @file_list) if $ex_data->is_dir;
 
         my $i2_root = create_second_instance ($app_to_test, $t_name, $wr_dir, $wr_dir2,$t, $config_dir_override);
 
+        load_instructions ($i2_root,$t->{load2},$trace) if $t->{load2} ;
+
         my $p2_dump = dump_tree("second $app_to_test", $i2_root, 'custom', $t->{no_warnings},{}) ;
 
         unified_diff;
@@ -515,6 +567,17 @@ sub translate_test_data {
     map {$t->{full_dump}{$_} = delete $t->{$_} if $t->{$_}; } qw/dump_warnings dump_errors/;
 }
 
+sub create_model_object {
+    my $new_model ;
+    eval { $new_model = Config::Model->new(); } ;
+    if ($@) {
+        # necessary to run smoke test (no Config::Model to avoid dependency loop)
+        plan skip_all => 'Config::Model is not loaded' ;
+        return;
+    }
+    return $new_model;
+}
+
 sub run_tests {
     my ( $arg, $test_only_app, $do ) = @_;
 
@@ -532,12 +595,6 @@ sub run_tests {
         Log::Log4perl->easy_init( $log ? $WARN : $ERROR );
     }
 
-    eval { $model = Config::Model->new(); } ;
-    if ($@) {
-        plan skip_all => 'Config::Model is not loaded' ;
-        return;
-    }
-
     Config::Model::Exception::Any->Trace(1) if $arg =~ /e/;
 
     ok( 1, "compiled" );
@@ -550,6 +607,8 @@ sub run_tests {
     foreach my $app_to_test_conf (@group_of_tests) {
         my ($app_to_test) = ( $app_to_test_conf =~ m!\.d/([\w\-]+)-test-conf! );
         next if ( $test_only_app and $test_only_app ne $app_to_test ) ;
+        $model = create_model_object();
+        return unless $model;
         run_model_test($app_to_test, $app_to_test_conf, $do, $model, $trace, $wr_root) ;
     }
 
@@ -572,7 +631,7 @@ Config::Model::Tester - Test framework for Config::Model
 
 =head1 VERSION
 
-version 2.057
+version 3.003
 
 =head1 SYNOPSIS
 
@@ -597,6 +656,20 @@ cases per model.
 
 A specific layout for test files must be followed.
 
+=head2 Test specification
+
+Each set of test is defined in a file like:
+
+ t/model_tests.d/<app-name>-test-conf.pl
+
+This file specifies that C<app-name> (which is defined in
+C<lib/Config/Model/*.d> directory) will be used for the test cases
+defined in the C<*-test-conf.pl> file.
+
+This file contains a list of test case (explained below) and expects a
+set of files used as test data. The layout of these test data files is
+explained in next section.
+
 =head2 Simple test file layout
 
 Each test case is represented by a configuration file (not
@@ -621,14 +694,14 @@ C<lib/Config/Model/system.d/lcdproc>
 Test specification is written in C<lcdproc-test-conf.pl> file (i.e. this
 modules looks for files named  like C<< <app-name>-test-conf.pl> >>).
 
-Subtests are specified in files in directory C<lcdproc-examples> (
-i.e. this modules looks for subtests in directory
-C<< <model-name>-examples.pl> >>. C<lcdproc-test-conf.pl> contains
+Subtests data is proviced in files in directory C<lcdproc-examples> (
+i.e. this modules looks for test data in directory
+C<< <model-name>-examples> >>. C<lcdproc-test-conf.pl> contains
 instructions so that each file will be used as a C</etc/LCDd.conf>
 file during each test case.
 
 C<lcdproc-test-conf.pl> can contain specifications for more test
-cases. Each test case will require a new file in C<lcdproc-examples>
+cases. Each test case requires a new file in C<lcdproc-examples>
 directory.
 
 See L</Examples> for a link to the actual LCDproc model tests
@@ -750,6 +823,15 @@ here and specify warning tests or warning filters as mentioned below.
 
 See actual L<fstab test|https://github.com/dod38fr/config-model/blob/master/t/model_tests.d/fstab-test-conf.pl>.
 
+=head2 Skip a test
+
+A test file can be skipped using C<$skip> global variable.
+
+In this example, test is skipped when not running on a Debian system:
+
+ eval { require AptPkg::Config; };
+ $skip = ( $@ or not -r '/etc/debian_version' ) ? 1 : 0;
+
 =head2 Internal tests or backend tests
 
 Some tests will require the creation of a configuration class dedicated
@@ -764,9 +846,11 @@ L<test for shellvar backend|https://github.com/dod38fr/config-model/blob/master/
 
 =head2 Test specification with arbitrary file names
 
-In some models (e.g. C<Multistrap>, the config file is chosen by the user.
-In this case, the file name must be specified for each tests case:
+In some models like C<Multistrap>, the config file is chosen by the
+user. In this case, the file name must be specified for each tests
+case:
 
+ # not needed if test file is named multistrap-test-conf.pl
  $model_to_test = "Multistrap";
 
  @tests = (
@@ -777,7 +861,35 @@ In this case, the file name must be specified for each tests case:
     },
  );
 
-See actual L<multistrap test|https://github.com/dod38fr/config-model/blob/master/t/model_tests.d/multistrap-test-conf.pl>.
+See the actual L<multistrap test|https://github.com/dod38fr/config-model/blob/master/t/model_tests.d/multistrap-test-conf.pl>.
+
+=head2 Backend argument
+
+Some application like systemd requires a backend argument specified by
+User (e.g. a service name for systemd). The parameter C<backend_arg>
+can be specified to emulate this behavior.
+
+=head2 Re-use test data
+
+When the input data for test is quite complex (several files), it may
+be interested to re-use these data for other tests case. Knowing that
+test name must must unique, you can re-use test data with C<data_from>
+parameter. For instance:
+
+  @tests = (
+    {
+        name  => 'some-test',
+        # ...
+    },
+    {
+        name  => 'some-other-test',
+        data_from  => 'some-test',    # re-use data from test above
+        # ...
+    },
+
+See
+L<plainfile backend test|https://github.com/dod38fr/config-model/blob/master/t/model_tests.d/backend-plainfile-test-conf.pl>
+for a real life example.
 
 =head2 Test scenario
 
@@ -811,11 +923,40 @@ Use an empty array_ref to mask load warnings.
 
 Optionally run L<update|App::Cme::Command::update> command:
 
-    update => { in => 'some-test-data.txt', returns => 'foo' , no_warnings => [ 0 | 1 ] }
+    update => {
+         [ returns => 'foo' , ]
+         no_warnings => [ 0 | 1 ], # default 0
+         quiet => [ 0 | 1], # default 0, passed to update method
+         update_warnings => [ qr/.../, ]
+ }
 
-C<returns> is the expected return value (optional). All other
-arguments are passed to C<update> method. Note that C<< quiet => 1 >>
-may be useful for less verbose test.
+Where:
+
+=over
+
+=item *
+
+C<returns> is the expected return value (optional).
+
+=item *
+
+C<no_warnings> to suppress the warnings coming from
+L<Config::Model::Value>. Note that C<< no_warnings => 1 >> may be
+useful for less verbose test.
+
+=item *
+
+C<quiet> to suppress progress messages during update.
+
+=item *
+
+C<update_warnings> is an array ref of quoted regexp (See qr operator)
+to check the warnings produced during update. use C<< update => [] >>
+to check that no warnings are issued during update.
+
+=back
+
+All other arguments are passed to C<update> method.
 
 =item *
 
@@ -964,6 +1105,20 @@ in an array ref:
 
 =item *
 
+Check the mode of the written files:
+
+  file_mode => {
+     "~/.ssh/ssh_config"     => 0600, # octal mode
+     "debian/stuff.postinst" => 0755,
+  }
+
+Only the last four octets of the mode are tested. I.e. the test is done with
+C< $file_mode & 07777 >
+
+Note: this test is skipped on Windows
+
+=item *
+
 Check added or removed configuration files. If you expect changes,
 specify a subref to alter the file list:
 
@@ -973,6 +1128,9 @@ specify a subref to alter the file list:
         push @$list_ref, "/debian/source/format" ;
     },
 
+Note that actual and expected file lists are sorted before check,
+adding a file can be done with C<push>.
+
 =item *
 
 Copy all config data from C<< wr_root/<subtest name>/ >>
@@ -982,7 +1140,7 @@ the original configuration.
 
 =item *
 
-Create another configuration instance to read the conf file that was just copied
+Create a second configuration instance to read the conf file that was just copied
 (configuration data is checked.)
 
 =item *
@@ -992,6 +1150,16 @@ some errors were ignored and cannot be fixed) with C<< load_check2 => 'no' >>
 
 =item *
 
+Optionally load configuration data in the second instance. You should
+design this config data to suppress any error or warning that occur in
+the step below. E.g:
+
+    load2 => 'binary:seaview',
+
+See L<Config::Model::Loader> for the syntax of the string accepted by C<load2> parameter.
+
+=item *
+
 Compare data read from original data.
 
 =item *
@@ -1104,7 +1272,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2013-2016 by Dominique Dumont.
+This software is Copyright (c) 2013-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
@@ -1151,7 +1319,7 @@ CPANTS
 
 The CPANTS is a website that analyzes the Kwalitee ( code metrics ) of a distribution.
 
-L<http://cpants.perl.org/dist/overview/Config-Model-Tester>
+L<http://cpants.cpanauthors.org/dist/Config-Model-Tester>
 
 =item *
 
